#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
xhtml2odt - XHTML to ODT XML transformation
===========================================

Copyright (C) 2009-2010 Aurelien Bompard
Copyright (C) 2015 Étienne Loks

This script can convert a wiki page to the OpenDocument Text (ODT) format,
standardized as ISO/IEC 26300:2006, and the native format of office suites such
as OpenOffice.org, KOffice, and others.

It uses a template ODT file which will be filled with the converted content of
the XHTML page.

Website: http://xhtml2odt.org

Inspired by the work on docbook2odt_, by Roman Fordinal

.. _docbook2odt: http://open.comsultia.com/docbook2odf/


Usage
-----

Call the script with the :option:`--help` option to see all the available
options.  The main options are:

.. cmdoption:: -i <file>, --input <file>

   The HTML file to read from.

.. cmdoption:: -o <file>, --output <file>

   The ODT file to export to (will be overwritten if already present).

.. cmdoption:: -t <file>, --template <file>

   The ODT file to use as a template (must be readable).

.. cmdoption:: -v

   Be verbose (enables logging)


The full help message is::

    Usage: xhtml2odt.py [options] -i input -o output -t template.odt

    Options:
      -h, --help            show this help message and exit
      -i FILE, --input=FILE
                            Read the html from this file
      -o FILE, --output=FILE
                            Location of the output ODT file
      -t FILE, --template=FILE
                            Location of the template ODT file
      -u URL, --url=URL     Use this URL for relative links
      -v, --verbose         Show what's going on
      --html-id=ID          Only export from the element with this ID
      --replace=KEYWORD     Keyword to replace in the ODT template (default is
                            ODT-INSERT)
      --cut-start=KEYWORD   Keyword to start cutting text from the ODT template
                            (default is ODT-CUT-START)
      --cut-stop=KEYWORD    Keyword to stop cutting text from the ODT template
                            (default is ODT-CUT-STOP)
      --top-header-level=LEVEL
                            Level of highest header in the HTML (default is 1)
      --img-default-width=WIDTH
                            Default image width (default is 8cm)
      --img-default-height=HEIGHT
                            Default image height (default is 6cm)
      --dpi=DPI             Screen resolution in Dots Per Inch (default is 96)
      --no-network          Do not download remote images


License
-------

GNU LGPL v2.1 or later: http://www.gnu.org/licenses/lgpl-2.1.html

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

Code
----
"""

import tempfile
import shutil
import re
import os
import sys
import zipfile
import urllib2
import urlparse
import hashlib
import mimetypes
from StringIO import StringIO
from optparse import OptionParser

try:
    import tidy
except:
    from tidylib import tidy_document as tidy

from lxml import etree
from PIL import Image

#pylint#: disable-msg=C0301,C0111

INSTALL_PATH = os.path.dirname(__file__)

INCH_TO_CM = 2.54
CHARSET = "utf-8"

__version__ = 0.1

class ODTExportError(Exception):
    """Base exception for ODT conversion errors"""
    pass

class HTMLFile(object):
    """
    This class contains the HTML document to convert to ODT. The HTML code
    will be run through Tidy to ensure that is is valid and well-formed
    XHTML.

    :ivar options: An OptionParser-result object containing the options for
        processing.
    :type options: OptionsParser-result object
    :ivar html: The HTML code.
    :type html: ``str``
    """

    def __init__(self, options):
        self.options = options
        self.html = ""

    def read(self):
        """
        Read the HTML file from :attr:`options`.input, run it through Tidy, and
        filter using the selected ID (if applicable).
        """
        in_file = open(self.options.input)
        self.html = in_file.read()
        in_file.close()
        self.cleanup()
        if self.options.htmlid:
            self.select_id()

    def cleanup(self):
        """
        Run the HTML code from the :attr:`html` instance variable through Tidy.
        """
        tidy_options = dict(output_xhtml=1, add_xml_decl=1, indent=1,
                            tidy_mark=0, #input_encoding=str(self.charset),
                            output_encoding='utf8', doctype='auto',
                            wrap=0, char_encoding='utf8')
        self.html = str(tidy.parseString(self.html, **tidy_options))
        if not self.html:
            raise ODTExportError(
                        "Tidy could not clean up the document, aborting.")
        # Replace nbsp with entity
        # http://www.mail-archive.com/analog-help@lists.meer.net/msg03670.html
        self.html = self.html.replace("&nbsp;", "&#160;")
        # Tidy creates newlines after <pre> (by indenting)
        self.html = re.sub('<pre([^>]*)>\n', '<pre\\1>', self.html)

    def select_id(self):
        """
        Replace the HTML content by an element in the content. The element
        is selected by its HTML ID.
        """
        try:
            html_tree = etree.fromstring(self.html)
        except etree.XMLSyntaxError:
            if self.options.verbose:
                raise
            else:
                raise ODTExportError("The XHTML is still not valid after "
                                     "Tidy's work, I can't convert it.")
        selected = html_tree.xpath("//*[@id='%s']" % self.options.htmlid)
        if selected:
            self.html = etree.tostring(selected[0], method="html")
        else:
            print >> sys.stderr, "Can't find the selected HTML id: %s, " \
                                 % self.options.htmlid \
                                +"converting everything."


class ODTFile(object):
    """Handles the conversion and production of an ODT file"""

    def __init__(self, options):
        self.options = options
        self.xml = {
            "content": "",
            "styles": "",
        }
        self.tmpdir = tempfile.mkdtemp(prefix="xhtml2odt-")
        self.zfile = None
        self._added_images = []

    def open(self):
        """
        Uncompress the template ODT file, and read the content.xml and
        styles.xml files into memory.
        """
        self.zfile = zipfile.ZipFile(self.options.template, "r")
        for name in self.zfile.namelist():
            fname = os.path.join(self.tmpdir, name)
            if not os.path.exists(os.path.dirname(fname)):
                os.makedirs(os.path.dirname(fname))
            if name[-1] == "/":
                if not os.path.exists(fname):
                    os.mkdir(fname)
                continue
            fname_h = open(fname, "w")
            fname_h.write(self.zfile.read(name))
            fname_h.close()
        for xmlfile in self.xml:
            self.xml[xmlfile] = self.zfile.read("%s.xml" % xmlfile)

    def import_xhtml(self, xhtml):
        """
        Main function to run the conversion process:

        * XHTML import
        * conversion to ODT XML
        * insertion into the ODT template
        * adding of the missing styles

        The next logical step is to use the :meth:`save` method.

        :param xhtml: the XHTML content to import
        :type  xhtml: str
        """
        odt = self.xhtml_to_odt(xhtml)
        self.insert_content(odt)
        self.add_styles()

    def xhtml_to_odt(self, xhtml):
        """
        Converts the XHTML content into ODT.

        :param xhtml: the XHTML content to import
        :type  xhtml: str
        :returns: the ODT XML from the conversion
        :rtype: str
        """
        xsl_dir = os.path.join(INSTALL_PATH, 'xsl')
        tmpdirxsl = os.path.join(tempfile.mkdtemp(prefix="xhtml2odt-"), 'xsl')
        shutil.copytree(xsl_dir, tmpdirxsl)

        xslt_doc = etree.parse(os.path.join(tmpdirxsl, "xhtml2odt.xsl"))
        transform = etree.XSLT(xslt_doc)
        xhtml = self.handle_images(xhtml)
        xhtml = self.handle_links(xhtml)
        try:
            xhtml = etree.fromstring(xhtml) # must be valid xml at this point
        except etree.XMLSyntaxError:
            if self.options.verbose:
                raise
            else:
                raise ODTExportError("The XHTML is still not valid after "
                                     "Tidy's work, I can't convert it.")
        params = {
            "url": "/",
            "heading_minus_level": str(self.options.top_header_level - 1),
        }
        if self.options.verbose:
            params["debug"] = "1"
        if self.options.img_width:
            if hasattr(etree.XSLT, "strparam"):
                params["img_default_width"] = etree.XSLT.strparam(
                                                self.options.img_width)
            else: # lxml < 2.2
                params["img_default_width"] = "'%s'" % self.options.img_width
        if self.options.img_height:
            if hasattr(etree.XSLT, "strparam"):
                params["img_default_height"] = etree.XSLT.strparam(
                                                self.options.img_height)
            else: # lxml < 2.2
                params["img_default_height"] = "'%s'" % self.options.img_height
        odt = transform(xhtml, **params)
        # DEBUG
        #print str(odt)
        return str(odt).replace('<?xml version="1.0" encoding="utf-8"?>','')

    def handle_images(self, xhtml):
        """
        Handling of image tags in the XHTML. Local and remote images are
        handled differently: see the :meth:`handle_local_img` and
        :meth:`handle_remote_img` methods for details.

        :param xhtml: the XHTML content to import
        :type  xhtml: str
        :returns: XHTML with normalized ``img`` tags
        :rtype: str
        """
        # Handle local images
        xhtml = re.sub('<img [^>]*src="([^"]+)"[^>]*>',
                      self.handle_local_img, xhtml)
        # Handle remote images
        if self.options.with_network:
            xhtml = re.sub('<img [^>]*src="(https?://[^"]+)"[^>]*>',
                          self.handle_remote_img, xhtml)
        #print xhtml
        return xhtml

    def handle_local_img(self, img_mo):
        """
        Handling of local images. This method should be called as a callback on
        each ``img`` tag.

        Find the real path of the image file and use the :meth:`handle_img`
        method to flag it for inclusion in the ODT file.

        This implementation downloads the files that come from the same domain
        as the XHTML document cames from, but server-based export plugins can
        just retrieve it from the local disk, using either the
        ``DOCUMENT_ROOT`` or any appropriate method (depending on the web
        application you're writing an export plugin for).

        :param img_mo: the match object from the `re.sub` callback
        """
        log("handling local image: %s" % img_mo.group(1), self.options.verbose)
        src = img_mo.group(1)
        if src.count("://") and not src.startswith("file://"):
            # This is an absolute link, don't touch it
            return img_mo.group()
        if src.startswith("file://"):
            filename = src[7:]
        elif src.startswith("/"):
            filename = src
        else: # relative link
            filename = os.path.join(os.path.dirname(self.options.input), src)
        if os.path.exists(filename):
            return self.handle_img(img_mo.group(), src, filename)
        if src.startswith("file://") or not self.options.url:
            # There's nothing we can do here
            return img_mo.group()
        newsrc = urlparse.urljoin(self.options.url, os.path.normpath(src))
        if not self.options.with_network:
            # Don't download it, just update the URL
            return img_mo.group().replace(src, newsrc)
        try:
            tmpfile = self.download_img(newsrc)
        except (urllib2.HTTPError, urllib2.URLError):
            log("Failed getting %s" % newsrc, self.options.verbose)
            return img_mo.group()
        ret = self.handle_img(img_mo.group(), src, tmpfile)
        os.remove(tmpfile)
        return ret

    def handle_remote_img(self, img_mo):
        """
        Downloads remote images to a temporary file and flags them for
        inclusion using the :meth:`handle_img` method.

        :param img_mo: the match object from the `re.sub` callback
        """
        log('handling remote image: %s' % img_mo.group(), self.options.verbose)
        src = img_mo.group(1)
        try:
            tmpfile = self.download_img(src)
        except (urllib2.HTTPError, urllib2.URLError):
            return img_mo.group()
        ret = self.handle_img(img_mo.group(), src, tmpfile)
        os.remove(tmpfile)
        return ret

    def download_img(self, src):
        """
        Downloads the given image to a temporary location.

        :param src: the URL to download
        :type  src: str
        """
        log('Downloading image: %s' % src, self.options.verbose)
        # TODO: proxy support
        remoteimg = urllib2.urlopen(src)
        tmpimg_fd, tmpfile = tempfile.mkstemp()
        tmpimg = os.fdopen(tmpimg_fd, 'w')
        tmpimg.write(remoteimg.read())
        tmpimg.close()
        remoteimg.close()
        return tmpfile

    def handle_img(self, full_tag, src, filename):
        """
        Imports an image into the ODT file.

        :param full_tag: the full ``img`` tag in the original XHTML document
        :type  full_tag: str
        :param src: the ``src`` attribute of the ``img`` tag
        :type  src: str
        :param filename: the path to the image file on the local disk
        :type  filename: str
        """
        log('Importing image: %s' % filename, self.options.verbose)
        if not os.path.exists(filename):
            raise ODTExportError('Image "%s" is not readable or does not exist'
                                 % filename)
        # TODO: generate a filename (with tempfile.mkstemp) to avoid weird
        # filenames. Maybe use img.format for the extension
        if not os.path.exists(os.path.join(self.tmpdir, "Pictures")):
            os.mkdir(os.path.join(self.tmpdir, "Pictures"))
        newname = ( hashlib.md5(filename).hexdigest()
                    + os.path.splitext(filename)[1] )
        shutil.copy(filename, os.path.join(self.tmpdir, "Pictures", newname))
        self._added_images.append(os.path.join("Pictures", newname))
        full_tag = full_tag.replace('src="%s"' % src,
                                    'src="Pictures/%s"' % newname)
        try:
            img = Image.open(filename)
        except IOError:
            log('Failed to identify image: %s' % filename,
                self.options.verbose)
        else:
            width, height = img.size
            log('Detected size: %spx x %spx' % (width, height),
                self.options.verbose)
            width_mo = re.search('width="([0-9]+)(?:px)?"', full_tag)
            height_mo = re.search('height="([0-9]+)(?:px)?"', full_tag)
            if width_mo and height_mo:
                log('Forced size: %spx x %spx.' % (width_mo.group(),
                        height_mo.group()), self.options.verbose)
                width = float(width_mo.group(1)) / self.options.img_dpi \
                            * INCH_TO_CM
                height = float(height_mo.group(1)) / self.options.img_dpi \
                            * INCH_TO_CM
                full_tag = full_tag.replace(width_mo.group(), "")\
                                   .replace(height_mo.group(), "")
            elif width_mo and not height_mo:
                newwidth = float(width_mo.group(1)) / \
                           float(self.options.img_dpi) * INCH_TO_CM
                height = height * newwidth / width
                width = newwidth
                log('Forced width: %spx. Size will be: %scm x %scm' %
                    (width_mo.group(1), width, height), self.options.verbose)
                full_tag = full_tag.replace(width_mo.group(), "")
            elif not width_mo and height_mo:
                newheight = float(height_mo.group(1)) / \
                            float(self.options.img_dpi) * INCH_TO_CM
                width = width * newheight / height
                height = newheight
                log('Forced height: %spx. Size will be: %scm x %scm' %
                    (height_mo.group(1), height, width), self.options.verbose)
                full_tag = full_tag.replace(height_mo.group(), "")
            else:
                width = width / float(self.options.img_dpi) * INCH_TO_CM
                height = height / float(self.options.img_dpi) * INCH_TO_CM
                log('Size converted to: %scm x %scm' % (height, width),
                        self.options.verbose)
            full_tag = full_tag.replace('<img',
                    '<img width="%scm" height="%scm"' % (width, height))
        return full_tag

    def handle_links(self, xhtml):
        """
        Turn relative links into absolute links using the :meth:`handle_links`
        method.
        """
        # Handle local images
        xhtml = re.sub('<a [^>]*href="([^"]+)"',
                      self.handle_relative_links, xhtml)
        return xhtml

    def handle_relative_links(self, link_mo):
        """
        Do the actual conversion of links from relative to absolute. This
        method is used as a callback by the :meth:`handle_links` method.
        """
        href = link_mo.group(1)
        if href.startswith("file://") or not self.options.url:
            # There's nothing we can do here
            return link_mo.group()
        if href.count("://"):
            # This is an absolute link, don't touch it
            return link_mo.group()
        log("handling relative link: %s" % href, self.options.verbose)
        newhref = urlparse.urljoin(self.options.url, os.path.normpath(href))
        return link_mo.group().replace(href, newhref)

    def insert_content(self, content):
        """
        Insert ODT XML content into the ``content.xml`` file, replacing the
        keywords if needed.

        :param content: ODT XML content to insert
        :type  content: str
        """
        if self.options.replace_keyword and \
            self.xml["content"].count(self.options.replace_keyword) > 0:
            self.xml["content"] = re.sub(
                    "<text:p[^>]*>" +
                    re.escape(self.options.replace_keyword)
                    +"</text:p>", content, self.xml["content"])
        else:
            self.xml["content"] = self.xml["content"].replace(
                '</office:text>',
                content + '</office:text>')
        # Cut unwanted text
        if self.options.cut_start \
                and self.xml["content"].count(self.options.cut_start) > 0 \
                and self.options.cut_stop \
                and self.xml["content"].count(self.options.cut_stop) > 0:
            self.xml["content"] = re.sub(
                    re.escape(self.options.cut_start)
                    + ".*" +
                    re.escape(self.options.cut_stop),
                    "", self.xml["content"])

    def add_styles(self):
        """
        Scans the ODT XML for used styles that would not be already included in
        the ODT template, and adds those missing styles.
        """
        xsl_dir = os.path.join(INSTALL_PATH, 'xsl')
        xslt_doc = etree.parse(os.path.join(xsl_dir, "styles.xsl"))
        transform = etree.XSLT(xslt_doc)
        contentxml = etree.fromstring(self.xml["content"])
        stylesxml = etree.fromstring(self.xml["styles"])
        params = {}
        if self.options.verbose:
            params["debug"] = "1"
        self.xml["content"] = str(transform(contentxml, **params))
        self.xml["styles"] = str(transform(stylesxml, **params))

    def update_manifest(self):
        manifest_path = os.path.join(self.tmpdir, "META-INF", "manifest.xml")
        if not os.path.exists(manifest_path):
            return
        manifest = etree.parse(manifest_path)
        manifest_root = manifest.getroot()
        manifest_ns = "urn:oasis:names:tc:opendocument:xmlns:manifest:1.0"
        for img in self._added_images:
            mime_type = mimetypes.guess_type(img, strict=False)[0]
            if mime_type is None:
                continue
            img_el = etree.SubElement(
                        manifest_root,
                        "{%s}file-entry" % manifest_ns,
                        {"{%s}media-type" % manifest_ns: mime_type,
                         "{%s}full-path" % manifest_ns: img,
                        })
        manifest.write(manifest_path)

    def compile(self):
        """
        Writes the in-memory ODT XML content and styles to the disk
        """
        # Store the new content
        for xmlfile in self.xml:
            xmlf = open(os.path.join(self.tmpdir, "%s.xml" % xmlfile), "w")
            xmlf.write(self.xml[xmlfile])
            xmlf.close()
        self.update_manifest()

    def _build_zip(self, document):
        """
        Zips the working directory into a :class:`zipfile.ZipFile` object

        :param document: where the :class:`ZipFile` will be stored
        :type  document: str or file-like object
        """
        newzf = zipfile.ZipFile(document, "w", zipfile.ZIP_DEFLATED)
        for root, dirs, files in os.walk(self.tmpdir):
            for cur_file in files:
                realpath = os.path.join(root, cur_file)
                to_skip = len(self.tmpdir) + 1
                internalpath = os.path.join(root[to_skip:], cur_file)
                newzf.write(realpath, internalpath)
        newzf.close()

    def save(self, output=None):
        """
        General method to save the in-memory content to an ODT file on the disk.

        If :attr:`output` is ``None``, the document is returned.

        :param output: where the document should be saved, see the :option:`-o`
            option.
        :type  output: str or file-like object or ``None``
        :returns: if output is None: the ODT document ; or else ``None``.
        """
        self.compile()
        if output:
            document = output
        else:
            document = StringIO()
        self._build_zip(document)
        shutil.rmtree(self.tmpdir)
        if not output:
            return document.getvalue()


def log(msg, verbose=False):
    """
    Simple method to log if we're in verbose mode (with the :option:`-v`
    option).
    """
    if verbose:
        sys.stderr.write(msg+"\n")

def get_options():
    """
    Parses the command-line options.
    """
    usage = "usage: %prog [options] -i input -o output -t template.odt"
    parser = OptionParser(usage=usage)
    parser.add_option("--version", dest="version", action="store_true",
                      help="Show the version and exit")
    parser.add_option("-i", "--input", dest="input", metavar="FILE",
                      help="Read the html from this file")
    parser.add_option("-o", "--output", dest="output", metavar="FILE",
                      help="Location of the output ODT file")
    parser.add_option("-t", "--template", dest="template", metavar="FILE",
                      help="Location of the template ODT file")
    parser.add_option("-u", "--url", dest="url",
                      help="Use this URL for relative links")
    parser.add_option("-v", "--verbose", dest="verbose",
                      action="store_true", default=False,
                      help="Show what's going on")
    parser.add_option("--html-id", dest="htmlid", metavar="ID",
                      help="Only export from the element with this ID")
    parser.add_option("--replace", dest="replace_keyword",
                      default="ODT-INSERT", metavar="KEYWORD",
                      help="Keyword to replace in the ODT template "
                      "(default is %default)")
    parser.add_option("--cut-start", dest="cut_start",
                      default="ODT-CUT-START", metavar="KEYWORD",
                      help="Keyword to start cutting text from the ODT "
                      "template (default is %default)")
    parser.add_option("--cut-stop", dest="cut_stop",
                      default="ODT-CUT-STOP", metavar="KEYWORD",
                      help="Keyword to stop cutting text from the ODT "
                      "template (default is %default)")
    parser.add_option("--top-header-level", dest="top_header_level",
                      type="int", default="1", metavar="LEVEL",
                      help="Level of highest header in the HTML "
                      "(default is %default)")
    parser.add_option("--img-default-width", dest="img_width",
                      metavar="WIDTH", default="8cm",
                      help="Default image width (default is %default)")
    parser.add_option("--img-default-height", dest="img_height",
                      metavar="HEIGHT", default="6cm",
                      help="Default image height (default is %default)")
    parser.add_option("--dpi", dest="img_dpi", type="int",
                      default=96, metavar="DPI", help="Screen resolution "
                      "in Dots Per Inch (default is %default)")
    parser.add_option("--no-network", dest="with_network",
                      action="store_false", default=True,
                      help="Do not download remote images")
    options, args = parser.parse_args()
    if options.version:
        print "xhtml2odt %s" % __version__
        sys.exit(0)
    if len(args) > 0:
        parser.error("illegal arguments: %s"% ", ".join(args))
    if not options.input:
        parser.error("No input provided")
    if not options.output:
        parser.error("No output provided")
    if not options.template:
        default_template = os.path.join(INSTALL_PATH, "template.odt")
        if os.path.exists(default_template):
            options.template = default_template
        else:
            parser.error("No ODT template provided")
    if not os.path.exists(options.input):
        parser.error("Can't find input file: %s" % options.input)
    if not os.path.exists(options.template):
        parser.error("Can't find template file: %s" % options.template)
    return options

def main():
    """
    Main function, called when the script is invoked on the command line.
    """
    options = get_options()
    try:
        htmlfile = HTMLFile(options)
        htmlfile.read()
        odtfile = ODTFile(options)
        odtfile.open()
        odtfile.import_xhtml(htmlfile.html)
        odtfile.save(options.output)
    except ODTExportError, ex:
        print >> sys.stderr, ex
        print >> sys.stderr, "Conversion failed."
        sys.exit(1)

if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2014-2015  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

from django.utils.translation import ugettext as _

# overload of translation of registration module
_(u"username")
_(u"email address")
_(u"Related item")



from django import template
from django.core.exceptions import PermissionDenied
from django.conf.urls.defaults import patterns, url
from django.contrib import admin
from django.contrib.admin import helpers
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404, render_to_response
from django.contrib.admin.util import unquote
from django.utils.text import capfirst
from django.utils.html import mark_safe
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode


class SimpleHistoryAdmin(admin.ModelAdmin):
    object_history_template = "simple_history/object_history.html"
    object_history_form_template = "simple_history/object_history_form.html"

    def get_urls(self):
        """Returns the additional urls used by the Reversion admin."""
        urls = super(SimpleHistoryAdmin, self).get_urls()
        admin_site = self.admin_site
        opts = self.model._meta
        info = opts.app_label, opts.module_name,
        history_urls = patterns("",
            url("^([^/]+)/history/([^/]+)/$",
                admin_site.admin_view(self.history_form_view),
                name='%s_%s_simple_history' % info),)
        return history_urls + urls

    def history_view(self, request, object_id, extra_context=None):
        "The 'history' admin view for this model."
        model = self.model
        opts = model._meta
        app_label = opts.app_label
        pk_name = opts.pk.attname
        history = getattr(model, model._meta.simple_history_manager_attribute)
        action_list = history.filter(**{pk_name: object_id})
        # If no history was found, see whether this object even exists.
        obj = get_object_or_404(model, pk=unquote(object_id))
        context = {
            'title': _('Change history: %s') % force_unicode(obj),
            'action_list': action_list,
            'module_name': capfirst(force_unicode(opts.verbose_name_plural)),
            'object': obj,
            'root_path': getattr(self.admin_site, 'root_path', None),
            'app_label': app_label,
            'opts': opts
        }
        context.update(extra_context or {})
        context_instance = template.RequestContext(request, current_app=self.admin_site.name)
        return render_to_response(self.object_history_template, context, context_instance=context_instance)

    def history_form_view(self, request, object_id, version_id):
        original_model = self.model
        original_opts = original_model._meta
        history = getattr(self.model, self.model._meta.simple_history_manager_attribute)
        model = history.model
        opts = model._meta
        pk_name = original_opts.pk.attname
        record = get_object_or_404(model, **{pk_name: object_id, 'history_id': version_id})
        obj = record.instance
        obj._state.adding = False

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        if request.method == 'POST' and '_saveas_new' in request.POST:
            return self.add_view(request, form_url='../add/')

        formsets = []
        ModelForm = self.get_form(request, obj)
        if request.method == 'POST':
            form = ModelForm(request.POST, request.FILES, instance=obj)
            if form.is_valid():
                form_validated = True
                new_object = self.save_form(request, form, change=True)
            else:
                form_validated = False
                new_object = obj
            prefixes = {}

            if form_validated:
                self.save_model(request, new_object, form, change=True)
                form.save_m2m()

                change_message = self.construct_change_message(request, form, formsets)
                self.log_change(request, new_object, change_message)
                return self.response_change(request, new_object)

        else:
            form = ModelForm(instance=obj)

        adminForm = helpers.AdminForm(form, self.get_fieldsets(request, obj),
            self.prepopulated_fields, self.get_readonly_fields(request, obj),
            model_admin=self)
        media = self.media + adminForm.media

        url_triplet = (self.admin_site.name, original_opts.app_label,
                            original_opts.module_name)
        context = {
            'title': _('Revert %s') % force_unicode(obj),
            'adminform': adminForm,
            'object_id': object_id,
            'original': obj,
            'is_popup': False,
            'media': mark_safe(media),
            'errors': helpers.AdminErrorList(form, formsets),
            'app_label': opts.app_label,
            'original_opts': original_opts,
            'changelist_url': reverse('%s:%s_%s_changelist' % url_triplet),
            'change_url': reverse('%s:%s_%s_change' % url_triplet, args=(obj.pk,)),
            'history_url': reverse('%s:%s_%s_history' % url_triplet, args=(obj.pk,)),
            # Context variables copied from render_change_form
            'add': False,
            'change': True,
            'has_add_permission': self.has_add_permission(request),
            'has_change_permission': self.has_change_permission(request, obj),
            'has_delete_permission': self.has_delete_permission(request, obj),
            'has_file_field': True,
            'has_absolute_url': False,
            'ordered_objects': opts.get_ordered_objects(),
            'form_url': '',
            'opts': opts,
            'content_type_id': ContentType.objects.get_for_model(self.model).id,
            'save_as': self.save_as,
            'save_on_top': self.save_on_top,
            'root_path': getattr(self.admin_site, 'root_path', None),
        }
        context_instance = template.RequestContext(request, current_app=self.admin_site.name)
        return render_to_response(self.object_history_form_template, context, context_instance)

    def save_model(self, request, obj, form, change):
        """
        Add the admin user to a special model attribute for reference after save
        """
        obj._history_user = request.user
        super(SimpleHistoryAdmin, self).save_model(request, obj, form, change)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Context records forms definitions
"""
from itertools import groupby

from django import forms
from django.conf import settings
from django.core import validators
from django.forms.formsets import formset_factory
from django.utils.translation import ugettext_lazy as _

from ishtar_common.models import valid_id
from archaeological_operations.models import Period, Parcel, Operation, \
    ArchaeologicalSite
import models

from ishtar_common import widgets
from ishtar_common.forms import FinalForm, FormSet, \
    reverse_lazy, get_form_selection, TableSelect
from ishtar_common.forms_common import get_town_field, SourceSelect
from archaeological_operations.forms import OperationSelect, ParcelField,\
    RecordRelationsForm as OpeRecordRelationsForm


class RecordSelect(TableSelect):
    label = forms.CharField(label=_(u"ID"), max_length=100)
    parcel__town = get_town_field()
    if settings.COUNTRY == 'fr':
        operation__code_patriarche = forms.IntegerField(
            label=_(u"Code PATRIARCHE"))
    operation__year = forms.IntegerField(label=_(u"Operation's year"))
    operation__operation_code = forms.IntegerField(
        label=_(u"Operation's number (index by year)"))
    archaeological_sites = forms.IntegerField(
        label=_("Archaelogical site"),
        widget=widgets.JQueryAutoComplete(
            reverse_lazy('autocomplete-archaeologicalsite'),
            associated_model=ArchaeologicalSite),
        validators=[valid_id(ArchaeologicalSite)])
    datings__period = forms.ChoiceField(label=_(u"Period"), choices=[])
    unit = forms.ChoiceField(label=_(u"Unit type"), choices=[])
    parcel = ParcelField(label=_(u"Parcel (section/number)"))
    relation_types = forms.MultipleChoiceField(
        label=_(u"Search within relations"), choices=[],
        widget=forms.CheckboxSelectMultiple)

    def __init__(self, *args, **kwargs):
        super(RecordSelect, self).__init__(*args, **kwargs)
        self.fields['datings__period'].choices = Period.get_types()
        self.fields['datings__period'].help_text = Period.get_help()
        self.fields['unit'].choices = models.Unit.get_types()
        self.fields['unit'].help_text = models.Unit.get_help()
        self.fields['relation_types'].choices = models.RelationType.get_types(
            empty_first=False)

    def get_input_ids(self):
        ids = super(RecordSelect, self).get_input_ids()
        ids.pop(ids.index('parcel'))
        ids.append('parcel_0')
        ids.append('parcel_1')
        ids.pop(ids.index('relation_types'))
        for idx, c in enumerate(self.fields['relation_types'].choices):
            ids.append('relation_types_{}'.format(idx))
        return ids


class RecordFormSelection(forms.Form):
    form_label = _("Context record search")
    associated_models = {'pk': models.ContextRecord}
    currents = {'pk': models.ContextRecord}
    pk = forms.IntegerField(
        label="", required=False,
        widget=widgets.JQueryJqGrid(
            reverse_lazy('get-contextrecord'),
            RecordSelect, models.ContextRecord,
            source_full=reverse_lazy('get-contextrecord-full')),
        validators=[valid_id(models.ContextRecord)])

    def clean(self):
        cleaned_data = self.cleaned_data
        if 'pk' not in cleaned_data or not cleaned_data['pk']:
            raise forms.ValidationError(_(u"You should at least select one "
                                          u"context record."))
        return cleaned_data


class RecordFormGeneral(forms.Form):
    form_label = _("General")
    associated_models = {'parcel': Parcel, 'unit': models.Unit}
    pk = forms.IntegerField(required=False, widget=forms.HiddenInput)
    operation_id = forms.IntegerField(widget=forms.HiddenInput)
    parcel = forms.ChoiceField(label=_("Parcel"), choices=[])
    label = forms.CharField(label=_(u"ID"),
                            validators=[validators.MaxLengthValidator(200)])
    description = forms.CharField(label=_(u"Description"),
                                  widget=forms.Textarea, required=False)
    length = forms.IntegerField(label=_(u"Length (cm)"), required=False)
    width = forms.IntegerField(label=_(u"Width (cm)"), required=False)
    thickness = forms.IntegerField(label=_(u"Thickness (cm)"), required=False)
    depth = forms.IntegerField(label=_(u"Depth (cm)"), required=False)
    unit = forms.ChoiceField(label=_("Unit"), required=False, choices=[])
    location = forms.CharField(
        label=_(u"Location"), widget=forms.Textarea,
        required=False, validators=[validators.MaxLengthValidator(200)])

    def __init__(self, *args, **kwargs):
        operation = None
        if 'data' in kwargs and kwargs['data'] and \
                ('operation' in kwargs['data'] or
                 'context_record' in kwargs['data']):
            if 'operation' in kwargs['data']:
                operation = kwargs['data']['operation']
            if 'context_record' in kwargs['data'] and \
               kwargs['data']['context_record']:
                operation = kwargs['data']['context_record'].operation
            # clean data if not "real" data
            # prefix_value = kwargs['prefix']
            if not [k for k in kwargs['data'].keys()
                    if k.startswith(kwargs['prefix']) and kwargs['data'][k]]:
                kwargs['data'] = None
                if 'files' in kwargs:
                    kwargs.pop('files')
        super(RecordFormGeneral, self).__init__(*args, **kwargs)
        self.fields['parcel'].choices = [('', '--')]
        if operation:
            self.fields['operation_id'].initial = operation.pk
            parcels = operation.parcels.all()
            sort = lambda x: (x.town.name, x.section)
            parcels = sorted(parcels, key=sort)
            for key, gparcels in groupby(parcels, sort):
                self.fields['parcel'].choices.append(
                    (" - ".join(key), [(parcel.pk, parcel.short_label)
                                       for parcel in gparcels])
                )
        self.fields['unit'].choices = models.Unit.get_types()
        self.fields['unit'].help_text = models.Unit.get_help()

    def clean(self):
        # manage unique context record ID
        cleaned_data = self.cleaned_data
        operation_id = cleaned_data.get("operation_id")
        label = cleaned_data.get("label")
        cr = models.ContextRecord.objects.filter(
            label=label, parcel__operation__pk=operation_id)
        if 'pk' in cleaned_data and cleaned_data['pk']:
            cr = cr.exclude(pk=cleaned_data['pk'])
        if cr.count():
            raise forms.ValidationError(_(u"This ID already exists for "
                                          u"this operation."))
        return cleaned_data


class DatingForm(forms.Form):
    form_label = _("Dating")
    base_model = 'dating'
    associated_models = {'dating_type': models.DatingType,
                         'quality': models.DatingQuality,
                         'period': models.Period}
    period = forms.ChoiceField(label=_("Period"), choices=[])
    start_date = forms.IntegerField(label=_(u"Start date"), required=False)
    end_date = forms.IntegerField(label=_(u"End date"), required=False)
    quality = forms.ChoiceField(label=_("Quality"), required=False, choices=[])
    dating_type = forms.ChoiceField(label=_("Dating type"), required=False,
                                    choices=[])

    def __init__(self, *args, **kwargs):
        super(DatingForm, self).__init__(*args, **kwargs)
        self.fields['dating_type'].choices = models.DatingType.get_types()
        self.fields['dating_type'].help_text = models.DatingType.get_help()
        self.fields['quality'].choices = models.DatingQuality.get_types()
        self.fields['quality'].help_text = models.DatingQuality.get_help()
        self.fields['period'].choices = Period.get_types()
        self.fields['period'].help_text = Period.get_help()


DatingFormSet = formset_factory(DatingForm, can_delete=True,
                                formset=FormSet)
DatingFormSet.form_label = _("Dating")


class RecordRelationsForm(OpeRecordRelationsForm):
    current_model = models.RelationType
    current_related_model = models.ContextRecord
    associated_models = {'right_record': models.ContextRecord,
                         'relation_type': models.RelationType}
    right_record = forms.ChoiceField(
        label=_(u"Context record"), choices=[], required=False)

    def __init__(self, *args, **kwargs):
        crs = None
        if 'data' in kwargs and 'CONTEXT_RECORDS' in kwargs['data']:
            crs = kwargs['data']['CONTEXT_RECORDS']
            # clean data if not "real" data
            prefix_value = kwargs['prefix'] + '-right_record'
            if not [k for k in kwargs['data'].keys()
                    if k.startswith(prefix_value) and kwargs['data'][k]]:
                kwargs['data'] = None
                if 'files' in kwargs:
                    kwargs.pop('files')
        super(RecordRelationsForm, self).__init__(*args, **kwargs)
        self.fields['relation_type'].choices = \
            models.RelationType.get_types()
        if crs:
            self.fields['right_record'].choices = [('', '-' * 2)] + crs

RecordRelationsFormSet = formset_factory(RecordRelationsForm, can_delete=True)
RecordRelationsFormSet.form_label = _(u"Relations")


class RecordFormInterpretation(forms.Form):
    form_label = _("Interpretation")
    associated_models = {'activity': models.ActivityType,
                         'identification': models.IdentificationType}
    has_furniture = forms.NullBooleanField(label=_(u"Has furniture?"),
                                           required=False)
    filling = forms.CharField(label=_(u"Filling"),
                              widget=forms.Textarea, required=False)
    interpretation = forms.CharField(label=_(u"Interpretation"),
                                     widget=forms.Textarea, required=False)
    activity = forms.ChoiceField(label=_(u"Activity"), required=False,
                                 choices=[])
    identification = forms.ChoiceField(label=_("Identification"),
                                       required=False, choices=[])
    taq = forms.IntegerField(label=_(u"TAQ"), required=False)
    taq_estimated = forms.IntegerField(label=_(u"Estimated TAQ"),
                                       required=False)
    tpq = forms.IntegerField(label=_(u"TPQ"), required=False)
    tpq_estimated = forms.IntegerField(label=_(u"Estimated TPQ"),
                                       required=False)

    def __init__(self, *args, **kwargs):
        super(RecordFormInterpretation, self).__init__(*args, **kwargs)
        self.fields['activity'].choices = models.ActivityType.get_types()
        self.fields['activity'].help_text = models.ActivityType.get_help()
        self.fields['identification'].choices = \
            models.IdentificationType.get_types()
        self.fields['identification'].help_text = \
            models.IdentificationType.get_help()

OperationRecordFormSelection = get_form_selection(
    'OperationRecordFormSelection', _(u"Operation search"), 'operation_id',
    Operation, OperationSelect, 'get-operation',
    _(u"You should select an operation."))


class RecordDeletionForm(FinalForm):
    confirm_msg = " "
    confirm_end_msg = _(u"Would you like to delete this context record?")

#########################################
# Source management for context records #
#########################################

SourceRecordFormSelection = get_form_selection(
    'SourceRecordFormSelection', _(u"Context record search"),
    'context_record', models.ContextRecord, RecordSelect, 'get-contextrecord',
    _(u"You should select a context record."))


class RecordSourceSelect(SourceSelect):
    context_record__parcel__town = get_town_field(
        label=_(u"Town of the operation"))
    context_record__operation__year = forms.IntegerField(
        label=_(u"Year of the operation"))
    context_record__datings__period = forms.ChoiceField(
        label=_(u"Period of the context record"), choices=[])
    context_record__unit = forms.ChoiceField(
        label=_(u"Unit type of the context record"), choices=[])

    def __init__(self, *args, **kwargs):
        super(RecordSourceSelect, self).__init__(*args, **kwargs)
        self.fields['context_record__datings__period'].choices = \
            Period.get_types()
        self.fields['context_record__datings__period'].help_text = \
            Period.get_help()
        self.fields['context_record__unit'].choices = models.Unit.get_types()
        self.fields['context_record__unit'].help_text = models.Unit.get_help()


RecordSourceFormSelection = get_form_selection(
    'RecordSourceFormSelection', _(u"Documentation search"), 'pk',
    models.ContextRecordSource, RecordSourceSelect, 'get-contextrecordsource',
    _(u"You should select a document."),
    get_full_url='get-contextrecordsource-full')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012-2016  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.core.exceptions import ObjectDoesNotExist
from django.core.files.storage import default_storage
from django.utils.translation import ugettext_lazy as _

from ishtar_common.forms import reverse_lazy
from ishtar_common.wizards import Wizard, DeletionWizard, SourceWizard
import models


class FindWizard(Wizard):
    file_storage = default_storage
    model = models.Find
    wizard_done_window = reverse_lazy('show-find')

    def get_current_contextrecord(self):
        step = self.steps.current
        if not step:
            return
        if step.endswith('_creation'):  # a context record has been selected
            main_form_key = 'selecrecord-' + self.url_name
            try:
                idx = int(self.session_get_value(main_form_key, 'pk'))
                current_cr = models.ContextRecord.objects.get(pk=idx)
                return current_cr
            except(TypeError, ValueError, ObjectDoesNotExist):
                pass
        current_item = self.get_current_object()
        if current_item:
            base_finds = current_item.base_finds.all()
            if base_finds:
                return base_finds[0].context_record

    def get_context_data(self, form, **kwargs):
        """
        Get the operation and context record "reminder" on top of wizard forms
        """
        context = super(FindWizard, self).get_context_data(form, **kwargs)
        current_cr = self.get_current_contextrecord()
        if not current_cr or self.steps.current.startswith('select-'):
            return context
        context['reminders'] = (
            (_(u"Operation"), unicode(current_cr.operation)),
            (_(u"Context record"), unicode(current_cr)))
        return context

    def get_extra_model(self, dct, form_list):
        dct = super(FindWizard, self).get_extra_model(dct, form_list)
        dct['order'] = 1
        if 'pk' in dct and type(dct['pk']) == models.ContextRecord:
            dct['base_finds__context_record'] = dct.pop('pk')
        return dct


class FindModificationWizard(FindWizard):
    modification = True
    filter_owns = {'selec-find_modification': ['pk']}


class FindDeletionWizard(DeletionWizard):
    model = models.Find
    fields = ['label', 'material_types', 'datings', 'find_number',
              'object_types', 'description', 'conservatory_state', 'mark',
              'preservation_to_considers', 'integrities', 'volume',
              'weight', 'length', 'width', 'height', 'diameter', 'comment']


class TreatmentWizard(Wizard):
    model = models.Treatment


class FindSourceWizard(SourceWizard):
    model = models.FindSource


class FindSourceDeletionWizard(DeletionWizard):
    model = models.FindSource
    fields = ['item', 'title', 'source_type', 'authors', ]


class TreatmentSourceWizard(SourceWizard):
    model = models.TreatmentSource



# force the retranslation of generated strings and external module
from django.utils.translation import ugettext as _
_(u"username")
_(u"email address")
_(u"warehouse")
_(u"New warehouse")
_(u"warehouse")
_(u"New organization")
_(u"New person")
_(u"New author")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _, ugettext

from ishtar_common.models import GeneralType, \
    LightHistorizedItem, OwnPerms, Address, Person


class WarehouseType(GeneralType):
    class Meta:
        verbose_name = _(u"Warehouse type")
        verbose_name_plural = _(u"Warehouse types")
        ordering = ('label',)


class Warehouse(Address, OwnPerms):
    name = models.CharField(_(u"Name"), max_length=40)
    warehouse_type = models.ForeignKey(WarehouseType,
                                       verbose_name=_(u"Warehouse type"))
    person_in_charge = models.ForeignKey(
        Person, on_delete=models.SET_NULL, related_name='warehouse_in_charge',
        verbose_name=_(u"Person in charge"), null=True, blank=True)
    comment = models.TextField(_(u"Comment"), null=True, blank=True)

    class Meta:
        verbose_name = _(u"Warehouse")
        verbose_name_plural = _(u"Warehouses")
        permissions = (
            ("view_warehouse", ugettext(u"Can view all Warehouses")),
            ("view_own_warehouse", ugettext(u"Can view own Warehouse")),
            ("add_own_warehouse", ugettext(u"Can add own Warehouse")),
            ("change_own_warehouse", ugettext(u"Can change own Warehouse")),
            ("delete_own_warehouse", ugettext(u"Can delete own Warehouse")),
        )

    def __unicode__(self):
        return u"%s (%s)" % (self.name, unicode(self.warehouse_type))


class ContainerType(GeneralType):
    length = models.IntegerField(_(u"Length (mm)"), blank=True, null=True)
    width = models.IntegerField(_(u"Width (mm)"), blank=True, null=True)
    height = models.IntegerField(_(u"Height (mm)"), blank=True, null=True)
    volume = models.IntegerField(_(u"Volume (l)"), blank=True, null=True)
    reference = models.CharField(_(u"Ref."), max_length=30)

    class Meta:
        verbose_name = _(u"Container type")
        verbose_name_plural = _(u"Container types")
        ordering = ('label',)


class Container(LightHistorizedItem):
    TABLE_COLS = ['reference', 'container_type', 'location']
    location = models.ForeignKey(Warehouse, verbose_name=_(u"Warehouse"))
    container_type = models.ForeignKey(ContainerType,
                                       verbose_name=_("Container type"))
    reference = models.CharField(_(u"Container ref."), max_length=40)
    comment = models.TextField(_(u"Comment"))

    class Meta:
        verbose_name = _(u"Container")
        verbose_name_plural = _(u"Containers")

    def __unicode__(self):
        lbl = u" - ".join((self.reference, unicode(self.container_type),
                           unicode(self.location)))
        return lbl



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2013-2015  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

import datetime
import unicodecsv

from django.conf import settings

from ishtar_common.data_importer import *
from ishtar_common.models import Person, OrganizationType

from archaeological_operations.data_importer import *

from archaeological_files import models


class ImportClosingFormater(ImportFormater):
    def post_process(self, obj, context, value, owner=None):
        value = self.formater.format(value)
        if not value:
            return
        open_date = obj.reception_date or obj.creation_date
        if not open_date:
            return
        obj.end_date = open_date + datetime.timedelta(30)
        obj.save()


class ImportMayorFormater(ImportFormater):
    def post_process(self, obj, context, value, owner=None):
        value = self.formater.format(value)
        if type(self.field_name) in (list, tuple):
            return  # not managed
        associated_obj = get_object_from_path(obj, self.field_name)
        if not value or not obj.main_town or not associated_obj:
            return
        if slugify(value).endswith('le-maire'):
            value += u" de " + obj.main_town.name
            value = value[:300]
        setattr(associated_obj, self.field_name.split('__')[-1], value)
        associated_obj.save()


class FilePostProcessing(object):
    def post_processing(self, item, data):
        if not item.end_date:  # auto-close
            open_date = item.reception_date or item.creation_date
            if open_date:
                item.end_date = open_date + datetime.timedelta(30)
        item.save()
        return item


class FileImporterSraPdL(FilePostProcessing, Importer):
    DESC = u"Exports dossiers SRA PdL : importeur Filemaker dossiers"
    SLUG = "sra-pdl-files"
    LINE_FORMAT = []
    OBJECT_CLS = models.File
    UNICITY_KEYS = ['external_id']
    DEFAULTS = {
        ('responsible_town_planning_service', 'attached_to'): {
            'organization_type': OrganizationType.objects.get(
                txt_idx="planning_service")},
        ('general_contractor', 'attached_to'): {
            'organization_type': OrganizationType.objects.get(
                txt_idx="general_contractor")},
        tuple(): {
            'file_type': models.FileType.objects.get(
                txt_idx='preventive'),
            'creation_date': datetime.datetime.now,
        },
        ('in_charge',): {'attached_to': None},  # initialized in __init__
    }

    def _init_line_format(self):
        tf = TownFormater()
        tf.town_dct_init()
        self.line_format = [
            None,  # A, 1
            ImportFormater(
                ['general_contractor__attached_to__address',  # B, 2
                 'general_contractor__attached_to__postal_code',
                 'general_contractor__attached_to__town'],
                [UnicodeFormater(500, clean=True),
                 UnicodeFormater(5, re_filter=RE_CD_POSTAL_FILTER),
                 UnicodeFormater(70, clean=True), ],
                regexp=RE_ADD_CD_POSTAL_TOWN,
                regexp_formater_args=[[0], [1], [2]], required=False,
                comment=u"Aménageur - adresse"),
            ImportMayorFormater(
                # C, 3 TODO - extraire nom_prenom_titre
                'general_contractor__raw_name',
                UnicodeFormater(200),
                comment=u"Aménageur - nom brut",
                post_processing=True,
                required=False),
            None,  # D, 4
            ImportFormater(
                "general_contractor__title",  # E, 5
                StrChoiceFormater(Person.TYPE, cli=True),
                required=False,
                comment=u"Aménageur - titre"),
            None,  # F, 6
            None,  # G, 7
            None,  # H, 8
            ImportFormater("parcels__year",  # I, 9
                           YearNoFuturFormater(),
                           required=False),
            # J, 10
            ImportParcelFormater('', required=False, post_processing=True),
            None,  # K, 11
            ImportFormater([['main_town', 'parcels__town']],  # L, 12
                           tf,
                           required=False,
                           comment=u"Commune (si non définie avant)"),
            ImportFormater([['main_town', 'parcels__town']],  # M, 13
                           tf,
                           required=False,
                           comment=u"Commune (si non définie avant)"),
            ImportFormater('saisine_type',  # N, 14
                           TypeFormater(models.SaisineType),
                           required=False,
                           comment=u"Type de saisine"),
            None,  # O, 15
            ImportFormater('name',  # P, 16
                           UnicodeFormater(),
                           comment=u"Nom du dossier",
                           required=False),
            None,  # Q, 17
            ImportFormater(
                [
                    'responsible_town_planning_service__raw_name',
                    # R, 18 service instructeur
                    'responsible_town_planning_service__attached_to__address',
                    'responsible_town_planning_service__'
                    'attached_to__postal_code',
                    'responsible_town_planning_service__attached_to__town'],
                [UnicodeFormater(300, clean=True),
                 UnicodeFormater(300, clean=True),
                 UnicodeFormater(5, re_filter=RE_CD_POSTAL_FILTER),
                 UnicodeFormater(70, clean=True), ],
                regexp=RE_NAME_ADD_CD_POSTAL_TOWN,
                regexp_formater_args=[[0], [1], [2], [3]],
                comment=u"Service instructeur - adresse",
                required=False),
            ImportFormater(
                'comment',  # S, 19
                UnicodeFormater(prefix=u'* Considérants : '),
                comment=u"Commentaire",
                concat=True,
                required=False),
            ImportYearFormater('reception_date',  # T, 20
                               DateFormater(['%d/%m/%Y', '%d/%m/%Y']),
                               comment=u"Date de réception",
                               required=False,
                               duplicate_fields=[['creation_date', False]]),
            None,  # U, 21
            None,  # V, 22
            None,  # W, 23
            None,  # X, 24
            None,  # Y, 25
            None,  # Z, 26
            None,  # AA, 27
            None,  # AB, 28
            None,  # AC, 29
            None,  # AD, 30
            None,  # AE, 31
            None,  # AF, 32
            None,  # AG, 33
            None,  # AH, 34
            ImportFormater('creation_date',  # AI, 35
                           DateFormater(['%d/%m/%Y', '%d/%m/%y']),
                           force_value=True,
                           comment=u"Date de création",
                           required=False,),
            None,  # AJ, 36
            ImportFormater('comment',  # AK, 37
                           UnicodeFormater(prefix=u"* Historique : "),
                           comment=u"Commentaire",
                           concat=True, required=False),
            ImportFormater('internal_reference',  # AL, 38
                           UnicodeFormater(60),
                           comment=u"Autre référence",
                           required=False),
            None,  # AM, 39
            None,  # AN, 40
            ImportFormater('comment',  # AO, 41
                           UnicodeFormater(
                               prefix=u"* Justificatif de prescription : "),
                           comment=u"Justificatif de prescription",
                           concat=True, required=False),
            ImportFormater('comment',  # AP, 42
                           UnicodeFormater(
                               prefix=u"* Justificatif d'intervention : "),
                           comment=u"Justificatif d'intervention",
                           concat=True, required=False),
            None,  # AQ, 43
            None,  # AR, 44
            None,  # AS, 45
            None,  # AT, 46
            ImportFormater('comment',  # AU, 47
                           UnicodeFormater(
                               prefix=u"* Méthodologie de l'opération : "),
                           comment=u"Méthodologie de l'opération",
                           concat=True, required=False),
            None,  # AV, 48
            ImportFormater('permit_reference',  # AW, 49
                           UnicodeFormater(300, clean=True),
                           regexp=RE_PERMIT_REFERENCE,
                           comment=u"Réf. du permis de construire",
                           required=False),
            ImportFormater('comment',  # AX, 50
                           UnicodeFormater(
                               prefix=u"* Référence de dossier aménageur : "),
                           comment=u"Référence de dossier aménageur",
                           concat=True, required=False),
            None,  # AY, 51
            None,  # AZ, 52
            ImportFormater('comment',  # BA, 53
                           UnicodeFormater(
                               prefix=u"* Numéro d'arrêté préfectoral : "),
                           comment=u"Numéro d'arrêté préfectoral",
                           concat=True, required=False),
            ImportFormater('comment',  # BB, 54
                           UnicodeFormater(
                               prefix=u"* Numéro d'arrêté SRA : "),
                           comment=u"Numéro d'arrêté SRA",
                           concat=True, required=False),
            ImportFormater('comment',  # BC, 55
                           UnicodeFormater(
                               prefix=u"* Numéro d'arrêté de "
                               u"post-diagnostic : "),
                           comment=u"Numéro d'arrêté de post-diagnostic",
                           concat=True, required=False),
            None,  # BD, 56
            ImportFormater([['main_town', 'parcels__town']],  # BE, 57
                           TownINSEEFormater(),
                           required=False,
                           comment=u"Commune (si non définie avant)"),
            ImportFormater('comment',  # BF, 58
                           UnicodeFormater(2000),
                           comment=u"Commentaire",
                           concat=True, required=False),
            None,  # BG, 59
            None,  # BH, 60
            None,  # BI, 61
            None,  # BJ, 62
            None,  # BK, 63
            None,  # BL, 64
            None,  # BM, 65
            None,  # BN, 66
            None,  # BO, 67
            None,  # BP, 68
            None,  # BQ, 69
            None,  # BR, 70
            None,  # BS, 71
            ImportFormater(  # BT, 72 nom service instructeur
                ['responsible_town_planning_service__attached_to__name', ],
                [UnicodeFormater(300, clean=True), ],
                regexp=RE_ORGA,
                comment=u"Service instructeur - nom",
                required=False),
            None,  # BU, 73
            None,  # BV, 74
            ImportFormater(
                'in_charge__raw_name',  # BW, 75 responsable
                UnicodeFormater(200),
                comment=u"Responsable - nom brut",
                required=False),
            ImportFormater('total_surface',  # BX, 76 surface totale
                           SurfaceFormater(),
                           comment=u"Surface totale",
                           required=False),
            ImportFormater('total_developed_surface',
                           # BY, 77 surface totale aménagée
                           SurfaceFormater(),
                           comment=u"Surface totale aménagée",
                           required=False),
            None,  # BZ, 78
            None,  # CA, 79
            None,  # CB, 80
            None,  # CC, 81
            None,  # CD, 82
            None,  # CE, 83
            None,  # CF, 84
            ImportFormater('permit_type',
                           TypeFormater(models.PermitType),
                           required=False,
                           comment=u"Type de permis"),  # CG, 85
            None,  # CH, 85
            ImportFormater('year',  # CI, 86
                           IntegerFormater(),
                           comment=u"Année du dossier",
                           required=True),
            ImportFormater('numeric_reference',  # CJ, 87
                           IntegerFormater(),
                           comment=u"Identifiant numérique",
                           required=True),
            ImportFormater('external_id',  # CK, 88
                           UnicodeFormater(),
                           comment=u"Identifiant externe",
                           required=True),
        ]

    def __init__(self, *args, **kwargs):
        super(FileImporterSraPdL, self).__init__(*args, **kwargs)
        self.DEFAULTS[('in_charge',)]['attached_to'], created = \
            models.Organization.objects.get_or_create(
                name='SRA Pays de la Loire',
                defaults={
                    'organization_type':
                    OrganizationType.objects.get(txt_idx='sra')})
        self._init_line_format()
        if tuple() not in self._defaults:
            self._defaults[tuple()] = {}
        self._defaults[tuple()]['history_modifier'] = self.history_modifier
        self._associate_db_target_to_formaters()


def test(filename):
    importer = FileImporterSraPdL(skip_lines=3, output='cli')
    with open(filename) as csv_file:
        encodings = [settings.ENCODING, settings.ALT_ENCODING, 'utf-8']
        for encoding in encodings:
            try:
                importer.importation(
                    [line for line in
                     unicodecsv.reader(csv_file, encoding='utf-8')])
                # importer.importation(unicode_csv_reader(
                #        [line.decode(encoding)
                #         for line in csv_file.readlines()])
                print importer.get_csv_errors()
                break
            except ImporterError, e:
                print(unicode(e))
                if e.type == ImporterError.HEADER \
                        and encoding != encodings[-1]:
                    csv_file.seek(0)
                    continue
            except UnicodeDecodeError:
                if encoding != encodings[-1]:
                    csv_file.seek(0)
                    continue



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2013 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf import settings
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _

from ishtar_common.utils import shortify

from menus import Menu

from ishtar_common.models import get_current_profile
from archaeological_operations.models import Operation
from archaeological_files.models import File
from archaeological_context_records.models import ContextRecord
from archaeological_finds.models import Find

profile = get_current_profile()
CURRENT_ITEMS = []
if profile.files:
    CURRENT_ITEMS.append((_(u"Archaeological file"), File))
CURRENT_ITEMS.append((_(u"Operation"), Operation))
if profile.context_record:
    CURRENT_ITEMS.append((_(u"Context record"), ContextRecord))
if profile.find:
    CURRENT_ITEMS.append((_(u"Find"), Find))


def get_base_context(request):
    dct = {'URL_PATH': settings.URL_PATH}
    try:
        dct["APP_NAME"] = Site.objects.get_current().name
    except Site.DoesNotExist:
        dct["APP_NAME"] = settings.APP_NAME
    dct["COUNTRY"] = settings.COUNTRY
    """
    if 'MENU' not in request.session or \
       request.session['MENU'].user != request.user:
        menu = Menu(request.user)
        menu.init()
        request.session['MENU'] = menu
    """  # temporary disabled
    current_action = None
    if 'CURRENT_ACTION' in request.session:
        dct['CURRENT_ACTION'] = request.session['CURRENT_ACTION']
        current_action = dct['CURRENT_ACTION']
    menu = Menu(request.user, current_action=current_action,
                session=request.session)
    menu.init()
    if menu.selected_idx is not None:
        dct['current_theme'] = "theme-%d" % (menu.selected_idx + 1)
    request.session['MENU'] = menu
    dct['MENU'] = request.session['MENU']
    dct['JQUERY_URL'] = settings.JQUERY_URL
    dct['JQUERY_UI_URL'] = settings.JQUERY_UI_URL
    dct['COUNTRY'] = settings.COUNTRY
    dct['current_menu'] = []
    for lbl, model in CURRENT_ITEMS:
        model_name = model.__name__.lower()
        cls = ''
        current = model_name in request.session and request.session[model_name]
        items = []
        for item in model.get_owns(request.user):
            pk = unicode(item.pk)
            if item.IS_BASKET:
                pk = "basket-" + pk
            selected = pk == current
            if selected:
                cls = item.get_short_menu_class()
            items.append((pk, shortify(unicode(item), 60),
                          selected, item.get_short_menu_class()))
        if items:
            dct['current_menu'].append((lbl, model_name, cls, items))
    return dct



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2013 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.forms import widgets
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _


class ParcelWidget(widgets.MultiWidget):
    def __init__(self, attrs=None):
        if not attrs:
            attrs = {'class': 'widget-parcel'}
        elif 'class' not in attrs:
            attrs['class'] = 'widget-parcel'
        else:
            attrs['class'] += ' widget-parcel'
        _widgets = (
            widgets.TextInput(attrs=attrs),
            widgets.TextInput(attrs=attrs),
        )
        super(ParcelWidget, self).__init__(_widgets, attrs)

    def decompress(self, value):
        if value:
            return value
        return [None, None]

    def format_output(self, rendered_widgets):
        return u' / '.join(rendered_widgets)


class SelectParcelWidget(widgets.TextInput):
    def render(self, *args, **kwargs):
        render = super(SelectParcelWidget, self).render(*args, **kwargs)
        render += u" <button name='formset_add' value='add'>%s</button>" \
            % _(u"Add")
        return mark_safe(render)



import unicodecsv
import datetime

from django.conf import settings

from ishtar_common.data_importer import Importer


def get_year(value):
    try:
        for fmt in ['%d/%m/%Y', '%d/%m/%Y']:
            return datetime.datetime.strptime(value, fmt).year
    except:
        pass

index_list = []


def treatment(data):
    internal_ref = data[37].strip()
    creation = data[34].strip()
    reception = data[19].strip()
    yr = get_year(creation)
    if not yr:
        yr = get_year(reception)

    idx, year = None, None
    if '-' in internal_ref:
        year, y_idx = internal_ref.split('-')
        if len(year) == 4:  # 2007-XXXX
            try:
                year = int(year)
                idx = int(y_idx)
            except ValueError:
                pass
    elif '.' in internal_ref:
        year, y_idx = internal_ref.split('.')
        if len(year) == 4:  # 2011.XXXX
            try:
                year = int(year)
                idx = int(y_idx)
            except ValueError:
                pass
    if not idx:
        idx = int(internal_ref)
    if year and year != yr:
        yr = year
    assert yr  # we should absolutly have a year!

    external_id = "{}{}-{}".format(settings.ISHTAR_LOCAL_PREFIX, yr, idx)
    assert (yr, external_id) not in index_list
    index_list.append((yr, external_id))
    return yr, idx, external_id


new_datas = []
with open('plouf.csv') as csv_file:
    datas = [line for line in unicodecsv.reader(csv_file,
                                                encoding='utf-8')]
    for idx, data in enumerate(datas):
        if idx < 3:
            # headers
            data.append('annee')
            data.append('identifiant numerique')
            data.append('external_id')
            new_datas.append(data)
            continue
        try:
            year, idx, external_id = treatment(data)
            data.append(year)
            data.append(idx)
            data.append(external_id)
            new_datas.append(data)
        except Exception as e:
            print("Line {}: {}".format(idx + 1, e))

csv = Importer()._get_csv(new_datas, empty=u'')
with open('plouf2.csv', 'w') as fle:
    fle.write(csv.encode('utf-8'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2013 Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Permission backend to manage "own" objects
"""

from django.contrib.auth.backends import ModelBackend
from django.core.exceptions import ObjectDoesNotExist
from django.db.models.loading import cache

import models


class ObjectPermBackend(ModelBackend):
    supports_object_permissions = True
    supports_anonymous_user = True

    def has_perm(self, user_obj, perm, model=None, obj=None, session=None):
        if not user_obj.is_authenticated():
            return False
        if not model:
            # let it manage by the default backend
            return False
        try:
            ishtar_user = models.IshtarUser.objects.get(user_ptr=user_obj)
        except ObjectDoesNotExist:
            return False
        try:
            is_ownperm = perm.split('.')[-1].split('_')[1] == 'own'
        except IndexError:
            is_ownperm = False
        if ishtar_user.has_right('administrator', session=session):
            return True
        main_right = ishtar_user.person.has_right(perm, session=session) \
            or user_obj.has_perm(perm)
        if not main_right or not is_ownperm:
            return main_right
        if obj is None:
            model_name = perm.split('_')[-1].lower()
            model = None
            for app in cache.get_apps():
                for modl in cache.get_models(app):
                    if modl.__name__.lower() == model_name:
                        model = modl
            if not model:
                return False
            return not is_ownperm or model.has_item_of(ishtar_user)
        return not is_ownperm or obj.is_own(user_obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from ishtar_common.data_importer import *

from archaeological_finds import models


class FindsImporterBibracte(Importer):
    DESC = u"Exports Bibracte : importeur pour l'onglet mobilier"
    OBJECT_CLS = models.BaseFind
    DEFAULTS = {}
    LINE_FORMAT = [
        # OA
        ImportFormater('context_record__operation__operation_code',
                       IntegerFormater(),),
        # external_id
        ImportFormater(
            'external_id', UnicodeFormater(120, notnull=True),
            duplicate_fields=[('find__external_id', False),
                              ('label', False),
                              ('find__label', False)]),
        # isolé ou non (si non isolé = lot)
        None,  # à corriger
        # ImportFormater(
        #     'is_isolated',
        #     StrToBoolean(choices={'lot': False, 'objet': True}),
        #     required=False),
        # ???
        None,
        # A voir
        None,
        # cf type
        None,
        # Type = sous classe de matériaux = Liste hiérarchique
        ImportFormater('find__material_types',
                       TypeFormater(models.MaterialType), required=False),
        # ???
        None,
        # lien avec contenant
        None,
        # = nombre
        ImportFormater('find__find_number', IntegerFormater(), required=False),
        # poids
        ImportFormater('find__weight', FloatFormater(), required=False),
        # unité (g par défault)
        ImportFormater('find__weight_unit',
                       StrChoiceFormater(models.WEIGHT_UNIT), required=False),
        # lien UE
        ImportFormater('context_record__external_id', UnicodeFormater(120),),
        # date decouverte
        ImportFormater('discovery_date', DateFormater(['%Y/%m/%d']),
                       required=False,),
        # lien parcelle (unique)
        None,
        # etat conservation
        ImportFormater('find__conservatory_state',
                       TypeFormater(models.ConservatoryState), required=False),
        # preservation_to_consider
        ImportFormater('find__preservation_to_considers',
                       TypeFormater(models.PreservationType), required=False),
        # comment
        ImportFormater('comment', UnicodeFormater(1000), required=False),
        # lien vers plusieurs chrono (voir gestion actuelle chrono)
        None,
        # ImportFormater('find__datings__period', TypeFormater(Period,
        #                                   many_split="&"), required=False),
        # topographic_localisation
        ImportFormater('topographic_localisation', UnicodeFormater(120),
                       required=False),
        # special_interest
        ImportFormater('special_interest', UnicodeFormater(120),
                       required=False),
        # description
        ImportFormater('description', UnicodeFormater(1000), required=False),
        # remontage
        None
    ]


class FindAltImporterBibracte(Importer):
    DESC = u"Exports Bibracte : importeur pour l'onglet prélèvement"
    OBJECT_CLS = models.BaseFind
    DEFAULTS = {}
    LINE_FORMAT = [
        # code OA
        ImportFormater('context_record__operation__operation_code',
                       IntegerFormater(),),
        # identifiant prelevement
        ImportFormater('external_id', UnicodeFormater(120, notnull=True),
                       duplicate_fields=[('find__external_id', False)]),
        # nature
        ImportFormater('find__material_types',
                       TypeFormater(models.MaterialType), required=False),
        # identifiant UE
        ImportFormater('context_record__external_id', UnicodeFormater(120),),
        # identifiant materiel
        None,
        # commentaire
        ImportFormater('comment', UnicodeFormater(1000), required=False),
    ]


class ImportTreatmentFormater(ImportFormater):
    def post_process(self, obj, context, value, owner=None):
        if obj.upstream_treatment.count():
            return
        ope_code = context['upstream_treatment'][
            'base_finds']['context_record']['operation']['operation_code']
        ope_code = int(ope_code)
        downstream = models.Find.objects.filter(
            external_id=value,
            base_finds__context_record__operation__operation_code=ope_code)
        if not downstream.count():
            return
        downstream = downstream.all()[0]
        downstream.upstream_treatment = obj
        downstream.save()
        upstream = downstream.duplicate(owner)
        upstream.downstream_treatment = obj
        upstream.save()
        return


class TreatmentImporterBibracte(Importer):
    DESC = u"Exports Bibracte : importeur pour l'onglet traitement"
    OBJECT_CLS = models.Treatment
    DEFAULTS = {}
    LINE_FORMAT = [
        # code OA
        ImportFormater(
            'upstream_treatment__base_finds__context_record__operation__'
            'operation_code',
            UnicodeFormater(120, notnull=True)),
        # identifiant
        ImportTreatmentFormater(
            'external_id',
            UnicodeFormater(120, notnull=True), post_processing=True),
        None,
        # traitement
        ImportFormater('treatment_type', TypeFormater(models.TreatmentType),),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

ISHTAR_DPTS = [44, 49, 53, 72, 85]

# DB key: (txt_idx, (key, label))
ISHTAR_OPE_TYPES = {
    (u'préventive', u'BAT'):(u'building_study',
                             u"Étude de bâti (préventif)"),
    (u'programmée', u'BAT'):(u'building_study_research',
                             u"Étude de bâti (programmé)"),
    (u'préventive', u'DIAG'):(u'arch_diagnostic',
                              u"Diagnostic archéologique (préventif)"),
    (u'programmée', u'DIAG'):(u'arch_diagnostic_research',
                              u"Diagnostic archéologique (programmé)"),
    (u'préventive', u'ED'):(u'documents_study',
                            u'Étude documentaire (préventif)'),
    (u'programmée', u'ED'):(u'documents_study_research',
                            u'Étude documentaire (programmé)'),
    (u'préventive', u'EV'):(u'evaluation',
                            u'Évaluation'),
    (u'préventive', u'Fo. Prév.'):(u'prev_excavation',
                                   u"Fouille archéologique préventive"),
    (u'préventive', u'FO'):(u'prev_excavation',
                            u"Fouille archéologique préventive"),
    (u'programmée', u'FO'):(u'prog_excavation',
                            u"Fouille archéologique programmée"),
    (u'programmée', u'FP'):(u'prog_excavation',
                            u"Fouille archéologique programmée"),
    (u'programmée', u'FPP'):(u'prog_excavation',
                             u"Fouille archéologique programmée pluriannuelle"),
    (u'programmée', u'PA'):(u'aerial_survey_research',
                            u"Prospection aérienne (programmée)"),
    (u'préventive', u'PA'):(u'aerial_survey',
                            u"Prospection aérienne (préventif)"),
    (u'programmée', u'PCR'):(u"collective_research_project",
                             u"Projet Collectif de Recherche"),
    (u'programmée', u'PI'):(u'inventory_survey_research',
                            u"Prospection inventaire (programmé)"),
    (u'préventive', u'PI'):(u'inventory_survey',
                            u"Prospection inventaire (préventif)"),
    (u'programmée', u'PR'):(u'survey_research',
                            u"Prospection (programmé)"),
    (u'préventive', u'PR'):(u'survey',
                            u"Prospection (préventif)"),
    (u'programmée', u'PT'):(u'thematic_survey',
                            u"Prospection thématique"),
    (u'programmée', u'RE'):(u'rock_art_survey',
                            u"Prospection avec relevé d'art rupestre"),
    (u'préventive', u'SD'):(u'sampling',
                            u"Sondage (programmé)"),
    (u'programmée', u'SD'):(u'sampling_research',
                            u"Sondage (préventif)"),
    (u'préventive', u'SP'):(u'prev_excavation',
                            u"Fouille archéologique préventive"),
    (u'préventive', u'SU'):(u'emergency_excavation',
                            u"Sauvetage urgent"),
}

ISHTAR_PERIODS = {
    u'MA':u'middle_age',
    u'IND':u'indetermined',
    u'CON':u'contemporan',
    u'MOD':u'modern',
    u'REC':u'recent_times',
    u'BMA':u'low_middle_age',
    u'MAC':u'classic_middle_age',
    u'HMA':u'high_middle_age',
    u'BAS':u'low_empire',
    u'HAU':u'high-empire',
    u'NRE':u'republic',
    u'GAL':u'gallo-roman',
    u'FE2':u'second_iron_age',
    u'FE1':u'first_iron_age',
    u'BRF':u'final_bronze_age',
    u'BRM':u'middle_bronze_age',
    u'BRA':u'old_bronze_age',
    u'FER':u'iron_age',
    u'BRO':u'bronze_age',
    u'PRO':u'protohistory',
    u'NEF':u'final_neolithic',
    u'NER':u'recent_neolithic',
    u'NEM':u'middle_neolithic',
    u'NEA':u'old_neolithic',
    u'NEO':u'neolithic',
    u'MER':u'recent_mesolithic',
    u'MEM':u'middle_mesolithic',
    u'MEA':u'old_mesolithic',
    u'MES':u'mesolithic',
    u'PAF':u'final_paleolithic',
    u'PAS':u'late_paleolithic',
    u'PAM':u'middle_paleolithic',
    u'PAA':u'ancien_paleolithic',
    u'PAL':u'paleolithic',
    u'':u'not_yet_documented',
}

ISHTAR_PERMIT_TYPES = {
    '':(u"NP", 'Non précisé'),
    u'permis de démolir':(u"PD", u"Permis de démolir"),
    u"autorisation":(u"AUT", u"Autorisation"),
    u"étude d'impact":(u"EI", u"Étude d'impact"),
    u"certificat d'urbanisme":(u"CU", u"Certificat d'urbanisme"),
    u'installation classée':(u"IC", u'Installation classée'),
    u'permis de construire':(u"PC", u'Permis de construire'),
    u'découverte fortuite':(u"DF", u"Découvert fortuite"),
    u'autre':(u"O", u"Autre"),
    u'autorisation de travaux':(u"AT", u"Autorisation de travaux"),
    u'autorisation de lotir':(u"AL", u"Autorisation de lotir"),
    u'étude préalable MH':(u"EMH", u"Étude préalable MH")
}

ISHTAR_DOC_TYPES = {
    'RF':u"Rapport final",
    'RI':u"Rapport intermédiaire",
    "undefined":u"Non précisé"
}

# attrs, convert[, relative col number, multi]
ISHTAR_OPE_COL_FORMAT = [
 [('code_patriarche',), 'parse_string'],
 None, # pass
 None, # pass
 None, # pass
 [('code_dracar',), 'parse_string_10'],
 None, # pass
 (('in_charge',), 'parse_name_surname'),
 None, # organisation - valeur par défaut si pas precise en colonne "V"
 (('common_name',), 'parse_ope_name'), # Nom générique
 (('periods',), 'parse_period_name', None, True),
 (('cira_rapporteur',), 'parse_name_surname'),
 (('operation_type',), 'parse_patriarche_operationtype'),
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # commentaire adresse - géré en colonne S
 None, # pass
 (('comment',), 'parse_comment_addr_nature', [16]),
 None, #  (('associated_file', 'permit_type'), 'parse_permittype'),
 None, #  (('associated_file', 'internal_reference'), 'parse_fileref'),
 (('in_charge__attached_to',), 'parse_orga', [7]),
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 (('start_date',), 'parse_date'),
 None, # pass
 None, # pass
 (('negative_result',), 'parse_bool'),
 None, # pass
 None, #  (('associated_file', 'year'), 'parse_yearref'),
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 (('year',), 'parse_yearref'),
 None, # pass
 (('administrative_act', 'signature_date',), 'parse_date'),
 (('cira_date',), 'parse_date'),
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 (('end_date',), 'parse_date'),
 None, # pass
 None, # pass
 None, # act_type
 (('administrative_act', 'act_type'), 'parse_admin_act_typ', [49]),
 (('administrative_act', 'ref_sra'), 'parse_fileref'),
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 None, # pass
 (('surface',), 'parse_ha'),
 None, # pass
 (('source', 'index'), 'parse_rapp_index'),
 (('source', 'reference'), 'parse_string'),
 None, # pass
 None, # pass
 (('source', 'internal_reference'), 'parse_string'),
 (('source', 'receipt_date'), 'parse_date'),
 None, # pass
 None, # pass
 (('source', 'source_type'), 'parse_doc_types'),
 None, # pass
 None, # pass
 None, # pass
 (('towns',), 'parse_insee', None, True),
 None, # pass
 (('parcels',), 'parse_parcels', [79]),
 None, # pass
 None, # pass
 None, # pass
 (('eas_number',), 'parse_string'),
 None, # pass
 (('source', 'title'), 'parse_string'),
 None, # pass
 None, # pass
]




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012-2013  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Utils: import archaelogical operation from a CSV file
"""

DELIMITER = ";"
QUOTECHAR = '"'

import datetime
import csv, codecs
import re

from django.conf import settings
from django.db import transaction
from django.template.defaultfilters import slugify

from archaeological_operations.utils import *

from django.contrib.auth.models import User
from ishtar_common.models import Town, Person, PersonType, OrganizationType, \
    Organization, SourceType
from archaeological_files.models import PermitType, File, FileType
from archaeological_operations.models import Operation, OperationType, Period, \
                            AdministrativeAct, ActType, OperationSource, Parcel

class Column:
    def __init__(self, col_models, format, associated_cols=None, multi=False):
        self.col_models, self.format = col_models, format
        self.associated_cols, self.multi = associated_cols, multi

def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, **kwargs):
    # csv.py doesn't do Unicode; encode temporarily as UTF-8:

    csv_reader = csv.reader(utf_8_encoder(unicode_csv_data),
                            dialect=dialect, **kwargs)
    for row in csv_reader:
        # decode UTF-8 back to Unicode, cell by cell:
        yield [unicode(cell, 'utf-8') for cell in row]

def utf_8_encoder(unicode_csv_data):
    for line in unicode_csv_data:
        yield line.encode('utf-8')

# attrs, convert
DEFAULT_OPE_COLS = [
 [], # numéro de dossier ?
 (('operation_type',), parse_operationtype),
 (('common_name',), unicode),
 (('in_charge', 'name'), unicode),
 (('in_charge', 'surname'), unicode),
 [], # État ?
 [], # Adresse ?
 [], # origine ?
 (('periods',), parse_period),
 [], # Programme ?
 [], # Rattach PC ?
 [], # vide
 (('administrative_act', 'ref_sra'), unicode),
 (('administrative_act', 'signature_date'), parse_date),
 (('start_date',), parse_date),
 (('excavation_end_date',), parse_date),
 (('year',), parse_year),
 [], # identification
 (('code_patriarche',), int),
 [], # X degré
 [], # Y degré
 [], # X saisi ?
 [], # Y saisi ?
 [], # georef
 [], # geometrie
 (('surface',), parse_surface),
]

_OPE_COLS = settings.ISHTAR_OPE_COL_FORMAT if settings.ISHTAR_OPE_COL_FORMAT \
                                          else DEFAULT_OPE_COLS

OPE_COLS = []
for cols in _OPE_COLS:
    if cols:
        OPE_COLS.append(Column(*cols))
    else:
        OPE_COLS.append(None)

def ope_postimportfix(ope, dct):
    changed = False
    if not ope.year:
        sd = dct.get('start_date')
        ed = dct.get('end_date')
        if sd:
            ope.year = sd.year
            changed = True
        elif ed:
            ope.year = ed.year
            changed = True
    if changed:
        ope.save()
    return ope

class BreakIt(Exception):
    pass

class RelatedClass:
    def __init__(self, key, cls, default_data={}, reverse_key=False,
                 unique_keys=[], extra_data=[], mandatory_fields=[],
                 multi=None):
        self.key, self.cls, self.default_data = key, cls, default_data
        self.reverse_key, self.unique_keys = reverse_key, unique_keys
        self.extra_data, self.multi = extra_data, multi
        self.mandatory_fields = mandatory_fields

    def create_object(self, data):
        for mandatory_field in self.mandatory_fields:
            if not data.get(mandatory_field):
                return None
        if self.unique_keys:
            unique_data = {}
            for k in self.unique_keys:
                unique_data[k] = data.pop(k)
            created = False
            filtr = unique_data.copy()
            q = None
            # check if all condition have a value
            if not [k for k in filtr if not filtr[k]]:
                q = self.cls.objects.filter(**unique_data)
            if q and q.count() > 1:
                obj = q.all()[0]
            else:
                unique_data['defaults'] = data
                try:
                    obj, created = self.cls.objects.get_or_create(**unique_data)
                except ValueError:
                    return None
            if not created:
                for k in unique_data['defaults']:
                    try:
                        setattr(obj, k, unique_data['defaults'][k])
                    except ValueError:
                        continue
                obj.save()
        else:
            obj = self.cls.objects.create(**data)
        return obj

    def create(self, item, data, attr=None):
        datas = data
        if not self.multi:
            datas = [data]
        objs = []
        for data in datas:
            if self.reverse_key:
                data[self.reverse_key] = item
                if self.reverse_key not in self.unique_keys:
                    self.unique_keys.append(self.reverse_key)
                obj = self.create_object(data)
            else:
                obj = getattr(item, attr)
                if not obj:
                    obj = self.create_object(data)
                    setattr(item, attr, obj)
                    item.save()
                else:
                    for k in data:
                        setattr(obj, k, data[k])
                    obj.save()
            objs.append(obj)
        if not self.multi:
            return objs[0]
        return objs

#@transaction.commit_manually
def import_operations_csv(values, col_defs=OPE_COLS, update=True, person=None,
                      stdout=None, lines=None):
    default_person = person or User.objects.order_by('pk').all()[0]
    RELATED_CLASSES = [
        RelatedClass('administrative_act', AdministrativeAct,
                     default_data={'history_modifier':default_person,
                                     'act_type':ActType.objects.get(
                                            txt_idx='excavation_order')
                                     },
                     reverse_key='operation',
                     unique_keys=['ref_sra']),
        #RelatedClass('associated_file', File,
        #             extra_data=['year'],
        #             default_data={'history_modifier':default_person,
        #                           'file_type':FileType.objects.get(
        #                                        txt_idx='undefined')},
        #             unique_keys=['internal_reference', 'year']),
        RelatedClass('source', OperationSource, reverse_key='operation',
                     unique_keys=['index']),
        RelatedClass('parcels', Parcel, reverse_key='operation',
                     unique_keys=['operation', 'town', 'section',
                                  'parcel_number'],
                     multi=True),
    ]
    RELATED_CLASSES_KEYS = dict([(rel_cls.key, rel_cls)
                                   for rel_cls in RELATED_CLASSES])
    related_classes = RELATED_CLASSES_KEYS
    _prepare_ope_types()

    ope_default = {'history_modifier':default_person}
    new_ops = 0
    error_ope, error_reversed, error_multis = [], [], []
    multi_keys = set([column.col_models[0]
                      for column in col_defs if column and column.multi])

    restrict_lines = []
    start_line, end_line = None, None
    if lines:
        if '-' not in lines:
            restrict_lines = [int(line) for line in lines.split(',')]
        else:
            start_line, end_line = lines.split('-')
            start_line, end_line = int(start_line), int(end_line)+1
    if start_line:
        values = list(values)[start_line:]
    if end_line:
        values = list(values)[:end_line+1]
    for line_idx, vals in enumerate(values):
        if restrict_lines:
            if line_idx > max(restrict_lines):
                break
            if line_idx not in restrict_lines:
                continue
        if start_line:
            line_idx = line_idx + start_line
        if stdout:
            stdout.write("\r* line %d" % (line_idx))
        if not line_idx:
            continue # remove header
        args = {}
        for col_idx, val in enumerate(vals):
            if len(col_defs) <= col_idx or not col_defs[col_idx]:
                continue
            col_def = col_defs[col_idx]
            attrs, typ = col_def.col_models, col_def.format
            extra_cols = col_def.associated_cols
            if not callable(typ):
                if typ.startswith('parse_string_'):
                    typ = _get_parse_string(int(typ.split('_')[-1]))
                else:
                    typ = globals()[typ]
            c_args = args
            for attr in attrs:
                if attr not in c_args:
                    c_args[attr] = {}
                c_args = c_args[attr]
            try:
                if not extra_cols:
                        v = typ(val)
                else:
                    arguments = [vals[col_number] for col_number in extra_cols]
                    if not [arg for arg in arguments if arg]:
                        continue
                    arguments += [default_person]
                    v = typ(val, *arguments)
            except:
                v = None
            if len(attrs) == 1:
                args[attrs[0]] = v
            elif len(attrs) == 2:
                args[attrs[0]][attrs[1]] = v
            elif len(attrs) == 3:
                args[attrs[0]][attrs[1]][attrs[2]] = v
        # manage exploded dates
        for k in args.keys():
            if '__year' in k:
                key = k[:-len('__year')]
                try:
                    v = datetime.datetime(args[k], args[key+'__month'],
                                          args[key+'__day'])
                    args[key] = v
                except:
                    pass
                args.pop(k)
                args.pop(key+'__month')
                args.pop(key+'__day')
        related_items = []
        multis = []
        attached_models = {}
        for k in args.keys():
            try:
                if k in related_classes:
                    rel_cls = related_classes[k]
                    cls, default = rel_cls.cls, rel_cls.default_data
                    reverse_key = rel_cls.reverse_key
                    values = None
                    if rel_cls.multi:
                        values = []
                        for v in args[k]:
                            v.update(default)
                            values.append(v)
                    else:
                        values = default.copy()
                        values.update(args[k])
                    exited = False
                    for extra in rel_cls.extra_data:
                        if not args.get(extra):
                            raise BreakIt
                        values[extra] = args[extra]
                    args.pop(k)
                    related_items.append((rel_cls, values, k))
                elif k in multi_keys:
                    multis.append((k, args[k]))
                    args.pop(k)
                elif '__' in k:
                    mod, value = k.split('__')
                    attached_models[(mod, value)] = args.pop(k)
            except BreakIt:
                args.pop(k)
                continue
        op = None
        if not args.get('operation_type'):
            #print "Pas d'operation_type"
            continue
        #transaction.commit()
        try:
            int(args['code_patriarche'])
        except ValueError:
            continue
        q = Operation.objects.filter(code_patriarche=args['code_patriarche'])
        if q.count():
            if not update:
                #print "Code patriarche existant"
                continue
            op = q.all()[0]
        # check
        if not args.get('year') and args.get('start_date'):
            args['year'] = args['start_date'].year
        # creation
        """
        print "args", args
        print "multis", multis
        print "attached_models", attached_models
        """
        if not op:
            args.update(ope_default)
            #if not args.get('operation_code'):
            #    args['operation_code'] = Operation.get_available_operation_code(
            #                                                    args['year'])
            #try:
            op = Operation.objects.create(**args)
            #op.save()
            new_ops += 1
            #except:
            #    error_ope.append((line_idx, args))
            #    transaction.rollback()
            #    continue
            #transaction.commit()
        else: # mise à jour
            try:
                for k in args:
                    if getattr(op, k):
                        continue
                    setattr(op, k, args[k])
                op.save()
            except:
                #transaction.rollback()
                continue
            #transaction.commit()
        try:
            for k, vals in multis:
                if not vals:
                    continue
                for v in vals:
                    getattr(op, k).add(v)
                    op.save()
        except:
            #transaction.rollback()
            error_multis.append((line_idx, multis))
        for attr, attached_attr in attached_models:
            field = getattr(op, attr)
            if field:
                setattr(field, attached_attr, attached_models[(attr,
                                                               attached_attr)])
                field.save()
        #transaction.commit()
        for rel_cls, data, attr in related_items:
            rel_cls.create(op, data, attr)
            #try:
            #    rel_cls.create(op, data, attr)
            #except:
            #    error_reversed.append((line_idx, data))
        ope_postimportfix(op, args)
        #transaction.commit()

    errors = []
    if error_ope:
        error = "Error while recording these operations:\n"
        for line_idx, args in error_ope:
            error += "line: " + str(line_idx) + " args: " + str(args) + '\n'
        errors.append(error)
    if error_multis:
        error = "Error while recording these multiples items attached to "\
                "operation:"
        for line_idx, args in error_multis:
            error += "line: " + str(line_idx) + " args: " + str(args) + '\n'
        errors.append(error)
    if error_reversed:
        error = "Error while recording these items that depend to operation:"
        for line_idx, args in error_reversed:
            error += "line: " + str(line_idx) + " args: " + str(args) + '\n'
        errors.append(error)
    #transaction.commit()
    return new_ops, errors

def import_from_csv(filename, update=True, col_defs=OPE_COLS,
                    person=None, stdout=None, lines=None):
    """
    Import from a CSV file.
    Return number of operation treated and errors.
    """
    try:
        values = unicode_csv_reader(codecs.open(filename, 'rb', "utf-8"),
                        delimiter=DELIMITER, quotechar=QUOTECHAR)
    except (IOError):
        return 0, [u"Incorrect CSV file."]

    new_ops, errors = import_operations_csv(values, col_defs=col_defs,
                                    update=update, person=person, stdout=stdout,
                                    lines=lines)
    return new_ops, errors



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010-2016  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import TemplateView
from django.views.generic.edit import CreateView, FormView

from ishtar_common.forms import FinalForm
from ishtar_common.forms_common import SourceForm, AuthorFormset, \
    SourceDeletionForm
from ishtar_common.models import IshtarUser
from archaeological_context_records.forms \
    import RecordFormSelection as RecordFormSelectionTable

from ishtar_common.views import get_item, show_item, revert_item, \
    get_autocomplete_generic, IshtarMixin, LoginRequiredMixin
from ishtar_common.wizards import SearchWizard

from wizards import *
from forms import *
import models

find_extra_keys = {
    'base_finds__cache_short_id':
        'base_finds__cache_short_id__icontains',
    'base_finds__cache_complete_id':
        'base_finds__cache_complete_id__icontains',
    'label':
        'label__icontains',
    'base_finds__context_record':
        'base_finds__context_record__pk',
    'base_finds__context_record__parcel__town':
        'base_finds__context_record__parcel__town',
    'base_finds__context_record__operation__year':
        'base_finds__context_record__operation__year__contains',
    'base_finds__context_record__operation':
        'base_finds__context_record__operation__pk',
    'archaeological_sites':
        'base_finds__context_record__operation__archaeological_sites__pk',
    'base_finds__context_record__operation__code_patriarche':
        'base_finds__context_record__operation__code_patriarche',
    'datings__period': 'datings__period__pk',
    'base_finds__find__description':
        'base_finds__find__description__icontains',
    'base_finds__batch': 'base_finds__batch',
    'basket': 'basket',
    'image': 'image__isnull'}

get_find = get_item(
    models.Find, 'get_find', 'find',
    reversed_bool_fields=['image__isnull'],
    base_request={'downstream_treatment__isnull': True},
    extra_request_keys=find_extra_keys.copy())

get_find_for_ope = get_item(
    models.Find, 'get_find', 'find',
    reversed_bool_fields=['image__isnull'],
    base_request={'downstream_treatment__isnull': True},
    extra_request_keys=find_extra_keys.copy(),
    own_table_cols=models.Find.TABLE_COLS_FOR_OPE)

show_findsource = show_item(models.FindSource, 'findsource')

get_findsource = get_item(
    models.FindSource, 'get_findsource', 'findsource',
    bool_fields=['duplicate'],
    extra_request_keys={
        'title': 'title__icontains',
        'description': 'description__icontains',
        'comment': 'comment__icontains',
        'additional_information': 'additional_information__icontains',
        'find__context_record__operation__year':
            'find__context_record__operation__year',
        'find__datings__period': 'find__datings__period__pk',
        'find__description': 'find__description__icontains',
    })
show_find = show_item(models.Find, 'find')
revert_find = revert_item(models.Find)

show_findbasket = show_item(models.FindBasket, 'findbasket')

find_creation_wizard = FindWizard.as_view([
    ('selecrecord-find_creation', RecordFormSelectionTable),
    ('find-find_creation', FindForm),
    ('dating-find_creation', DatingFormSet),
    ('final-find_creation', FinalForm)],
    label=_(u"New find"),
    url_name='find_creation',)

find_search_wizard = SearchWizard.as_view([
    ('general-find_search', FindFormSelection)],
    label=_(u"Find search"),
    url_name='find_search',)

find_modification_wizard = FindModificationWizard.as_view([
    ('selec-find_modification', FindFormSelection),
    ('selecrecord-find_modification', RecordFormSelection),
    ('find-find_modification', FindForm),
    ('dating-find_modification', DatingFormSet),
    ('final-find_modification', FinalForm)],
    label=_(u"Find modification"),
    url_name='find_modification',)


def find_modify(request, pk):
    # view = find_modification_wizard(request)
    FindModificationWizard.session_set_value(
        request, 'selec-find_modification', 'pk', pk, reset=True)
    return redirect(
        reverse('find_modification',
                kwargs={'step': 'find-find_modification'}))

find_deletion_wizard = FindDeletionWizard.as_view([
    ('selec-find_deletion', FindFormSelection),
    ('final-find_deletion', FindDeletionForm)],
    label=_(u"Find deletion"),
    url_name='find_deletion',)


find_source_creation_wizard = FindSourceWizard.as_view([
    ('selec-find_source_creation', SourceFindFormSelection),
    ('source-find_source_creation', SourceForm),
    ('authors-find_source_creation', AuthorFormset),
    ('final-find_source_creation', FinalForm)],
    label=_(u"Find: new source"),
    url_name='find_source_creation',)

find_source_modification_wizard = FindSourceWizard.as_view([
    ('selec-find_source_modification', FindSourceFormSelection),
    ('source-find_source_modification', SourceForm),
    ('authors-find_source_modification', AuthorFormset),
    ('final-find_source_modification', FinalForm)],
    label=_(u"Find: source modification"),
    url_name='find_source_modification',)

find_source_deletion_wizard = FindSourceDeletionWizard.as_view([
    ('selec-find_source_deletion', FindSourceFormSelection),
    ('final-find_source_deletion', SourceDeletionForm)],
    label=_(u"Find: source deletion"),
    url_name='find_source_deletion',)

autocomplete_objecttype = get_autocomplete_generic(models.ObjectType)
autocomplete_materialtype = get_autocomplete_generic(models.MaterialType)
autocomplete_preservationtype = get_autocomplete_generic(
    models.PreservationType)
autocomplete_integritytype = get_autocomplete_generic(models.IntegrityType)


class NewFindBasketView(IshtarMixin, LoginRequiredMixin, CreateView):
    template_name = 'ishtar/form.html'
    model = models.FindBasket
    form_class = NewFindBasketForm
    page_name = _(u"New basket")

    def get_form_kwargs(self):
        kwargs = super(NewFindBasketView, self).get_form_kwargs()
        kwargs['user'] = IshtarUser.objects.get(pk=self.request.user.pk)
        return kwargs

    def get_success_url(self):
        return reverse('select_itemsinbasket',
                       kwargs={'pk': self.object.pk})

    def form_valid(self, form):
        self.object = form.save()
        return HttpResponseRedirect(self.get_success_url())


class SelectBasketForManagement(IshtarMixin, LoginRequiredMixin, FormView):
    template_name = 'ishtar/form.html'
    form_class = SelectFindBasketForm
    page_name = _(u"Manage items in basket")

    def get_form_kwargs(self):
        kwargs = super(SelectBasketForManagement, self).get_form_kwargs()
        kwargs['user'] = IshtarUser.objects.get(pk=self.request.user.pk)
        if 'pk' in self.kwargs:
            kwargs['initial'].update({'basket': self.kwargs['pk']})
        return kwargs

    def get_success_url(self, basket):
        return reverse('select_itemsinbasket',
                       kwargs={'pk': basket})

    def form_valid(self, form):
        return HttpResponseRedirect(self.get_success_url(
            form.cleaned_data['basket']))


class SelectItemsInBasket(IshtarMixin, LoginRequiredMixin, TemplateView):
    template_name = 'ishtar/manage_basket.html'
    page_name = _(u"Manage basket")

    def get_context_data(self, *args, **kwargs):
        context = super(SelectItemsInBasket, self).get_context_data(
            *args, **kwargs)
        self.user = IshtarUser.objects.get(pk=self.request.user.pk)
        try:
            self.basket = models.FindBasket.objects.get(
                pk=self.kwargs['pk'], user=self.user)
        except models.FindBasket.DoesNotExist:
            raise PermissionDenied
        context['basket'] = self.basket
        context['form'] = MultipleFindFormSelection()
        context['add_url'] = reverse('add_iteminbasket')
        context['list_url'] = reverse('list_iteminbasket',
                                      kwargs={'pk': self.basket.pk})
        return context

    def form_valid(self, form):
        return HttpResponseRedirect(self.get_success_url())


class FindBasketAddItemView(IshtarMixin, LoginRequiredMixin, FormView):
    template_name = 'ishtar/simple_form.html'
    form_class = FindBasketAddItemForm

    def get_success_url(self, basket):
        return reverse('list_iteminbasket', kwargs={'pk': basket.pk})

    def form_valid(self, form):
        user = IshtarUser.objects.get(pk=self.request.user.pk)
        # rights are checked on the form
        basket = form.save(user)
        return HttpResponseRedirect(self.get_success_url(basket))


class FindBasketListView(IshtarMixin, LoginRequiredMixin, TemplateView):
    template_name = 'ishtar/basket_list.html'

    def get_context_data(self, *args, **kwargs):
        context = super(FindBasketListView, self).get_context_data(
            *args, **kwargs)
        self.user = IshtarUser.objects.get(pk=self.request.user.pk)
        try:
            self.basket = models.FindBasket.objects.get(
                pk=self.kwargs['pk'], user=self.user)
        except models.FindBasket.DoesNotExist:
            raise PermissionDenied
        context['basket'] = self.basket
        context['item_url'] = '/'.join(
            reverse(models.Find.SHOW_URL, args=[1]).split('/')[:-1])
        context['delete_url'] = '/'.join(
            reverse('delete_iteminbasket', args=[1, 1]).split('/')[:-3])
        return context


class FindBasketDeleteItemView(IshtarMixin, LoginRequiredMixin, TemplateView):
    template_name = 'ishtar/simple_form.html'

    def get_success_url(self, basket):
        return reverse('list_iteminbasket', kwargs={'pk': basket.pk})

    def get(self, *args, **kwargs):
        user = self.request.user
        ishtaruser = IshtarUser.objects.get(pk=self.request.user.pk)
        try:
            find = models.Find.objects.get(
                pk=self.kwargs['find_pk'])
        except models.Find.DoesNotExist:
            raise PermissionDenied
        try:
            basket = models.FindBasket.objects.get(
                pk=self.kwargs['basket'], user=ishtaruser)
        except models.FindBasket.DoesNotExist:
            raise PermissionDenied
        if not user.is_superuser and \
                not ishtaruser.has_right('change_find') and \
                not (ishtaruser.has_right('change_own_find')
                     and find.is_own(user)):
            raise PermissionDenied
        basket.items.remove(find)
        return HttpResponseRedirect(self.get_success_url(basket))


class DeleteFindBasketView(IshtarMixin, LoginRequiredMixin, FormView):
    template_name = 'ishtar/form_delete.html'
    form_class = DeleteFindBasketForm
    success_url = '/'
    page_name = _(u"Delete basket")

    def get_form_kwargs(self):
        kwargs = super(DeleteFindBasketView, self).get_form_kwargs()
        kwargs['user'] = IshtarUser.objects.get(pk=self.request.user.pk)
        return kwargs

    def form_valid(self, form):
        form.save()
        return HttpResponseRedirect(self.get_success_url())

"""
treatment_creation_wizard = TreatmentWizard.as_view([
    ('basetreatment-treatment_creation', BaseTreatmentForm),
    ('selecfind-treatment_creation', UpstreamFindFormSelection),
    ('multiselecfinds-treatment_creation', FindMultipleFormSelection),
    ('container-treatment_creation', ContainerForm),
    ('resultfind-treatment_creation', ResultFindForm),
    ('resultfinds-treatment_creation', ResultFindFormSet),
    ('final-treatment_creation', FinalForm)],
    condition_dict={
        'selecfind-treatment_creation':
            check_treatment('basetreatment-treatment_creation',
                          'treatment_type', not_type_list=['physical_grouping',
                                                           'packaging']),
        'multiselecfinds-treatment_creation':
            check_treatment('basetreatment-treatment_creation',
                            'treatment_type', ['physical_grouping',
                                               'packaging']),
        'resultfinds-treatment_creation':
            check_treatment('basetreatment-treatment_creation',
                            'treatment_type', ['split']),
        'resultfind-treatment_creation':
            check_treatment('basetreatment-treatment_creation',
                            'treatment_type', not_type_list=['split']),
        'container-treatment_creation':
            check_treatment('basetreatment-treatment_creation',
                            'treatment_type', ['packaging']),
    },
    label=_(u"New treatment"),
    url_name='treatment_creation',)
"""

"""
treatment_source_creation_wizard = TreatmentSourceWizard.as_view([
    ('selec-treatment_source_creation', SourceTreatmentFormSelection),
    ('source-treatment_source_creation', SourceForm),
    ('authors-treatment_source_creation', AuthorFormset),
    ('final-treatment_source_creation', FinalForm)],
    url_name='treatment_source_creation',)

"""
